import logging
import subprocess
import time

import sau

def exec_cmd(cmd, timeout=600, env = None):
    log = logging.getLogger(sau.LOGNAME)
    log.debug('Executing "{}"'.format(' '.join(cmd)))
    proc = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            env = env)
    try:
        out, err = proc.communicate(timeout=timeout)
    except subprocess.TimeoutExpired as err:
        log.error('Command "{}" timed out, killing it.'.format(' '.join(cmd)))
        proc.kill()
        time.sleep(30)
        if proc.poll() != None:
            log.error('Command "{}" would not be killed, forcing a termination'.format(' '.join(cmd)))
            proc.terminate()
            time.sleep(5)

    return (proc.returncode, out.decode('utf-8'), err.decode('utf-8'))


def version_diff(new, old):
    """ This will return 100 if the versions are not properly compareable,
    otherwise it will return the number of points that differ. Typically 3 is a
    major upgrade, 2 minor, 1 point release and patch-levels. It can also
    return 0 if both versions are the same."""
    new_list = list(new.rsplit('.'))
    old_list = list(old.rsplit('.'))
    common = min(len(new_list), len(old_list))
    longest = max(len(new_list), len(old_list))
    diff = longest-common
    for i in range(0, common):
        if new_list[i] == old_list[i]:
            continue
        return len(new_list) - i + diff
    return 0

